<?php

namespace TeamChat\SDK\Activities;

use stdClass;

/**
 * Description of ActivityFatory.
 *
 * @author Team Chat <support@teamchat.com>
 */
class ActivityFactory
{
    /**
     * Get a concrete incoming message instance.
     *
     * @param stdClass $raw Raw representation of the message as obtained from wire
     *
     * @return Activity An event of a specific type
     */
    public static function create(stdClass $raw, $botId)
    {
        switch ($raw->type) {
            case Activity::TYPE_WELCOME:
                return new Welcome($raw);
            case Activity::TYPE_MESSAGE:
            case Activity::TYPE_SMART_MESSAGE:
                return new Message($raw);
            case Activity::TYPE_KICKED:
                return new MemberRemoved($raw);
            case Activity::TYPE_LEFT:
                return new MemberLeft($raw);
            case Activity::TYPE_JOINED:
                return new MemberAdded($raw);
            default:
                return new Activity($raw);
        }
    }
}
