<?php

namespace TeamChat\SDK;

use TeamChat\SDK\Messages\Chatlets\IncomingChatlet;
use TeamChat\SDK\Activities\Activity;
use TeamChat\SDK\Client\IAPIClient;
use TeamChat\SDK\Data\UserDataInterface;

/**
 * Description of Context.
 *
 * @author Team Chat <support@teamchat.com>
 */
class Context implements ContextInterface
{
    /**
     * @var Activity
     */
    protected $activity;
    /**
     * @var IAPIClient
     */
    protected $client;
    /**
     * @var UserDataInterface
     */
    protected $udata;

    public function __construct(Activity $activity, IAPIClient $client, UserDataInterface $udata)
    {
        $this->activity = $activity;
        $this->client = $client;
        $this->udata = $udata;
    }

    public function byId($id)
    {
        return new Room($id);
    }

    public function chatletByAlias($alias)
    {
        return;
    }

    public function chatletById($id)
    {
        return new IncomingChatlet(json_encode(array('formid' => $id)));
    }

    public function chatletByRef($ref)
    {
        //$this->client->httpclient->getChatletById($ref);
        return;
    }

    public function create()
    {
        return new Room(null);
    }

    public function createFlowlet()
    {
        return;
    }

    public function currentChatlet()
    {
        return new IncomingChatlet($this->activity->raw->text, $this->currentChatletAlias());
    }

    public function currentChatletAlias()
    {
        $aliases = array();
        $textObj = json_decode($this->activity->raw->text);
        if (!empty($textObj) && property_exists($textObj, 'formid')) {
            $formid = $textObj->formid;
            $aliases = $this->udata->getAlias($formid);
        }

        return $aliases;
    }

    public function currentComment()
    {
        $textObj = json_decode($this->activity->raw->text);
        try {
            if ($textObj->type == 'primary' && $textObj->latestAction == 'comment') {
                $c = new \stdClass();
                $comments = $textObj->comments->details->resps;
                $comment = array_pop($comments);
                $c->formid = $textObj->formid;
                $c->sender = $comment->user->userid;
                $c->senderName = $comment->user->username;
                $c->senderEmail = $comment->user->useremail;
                $c->timestamp = $comment->timestamp;
                $c->actionreplace = $textObj->actionreplace;
                $c->text = $comment->values->text;

                return new Messages\Comment($c);
            }
        } catch (Exception $ex) {
        }
    }

    public function currentEvent()
    {
        return $this->activity;
    }

    public function currentFlowlet()
    {
        return;
    }

    public function currentKeywordParams()
    {
        return;
    }

    protected function extractKeys($action)
    {
        $keylist = array();
        $entries = get_object_vars($action);
        foreach ($entries as $entry) {
            $fields = json_decode($entry->formjson)->fields;
            foreach ($fields as $field) {
                if ($field->type != 'label') {
                    $keylist[] = $field->name;
                }
            }
        }

        return $keylist;
    }

    public function currentReply()
    {
        $textObj = json_decode($this->activity->raw->text);
        try {
            if ($textObj->type === 'locform') {
                $respId = 'resp' + $textObj->numresp;
                $reply = $textObj->mresps->{$respId};

                $r = new \stdClass();

                $r->formid = $textObj->formid;
                $r->sender = $reply->userid;
                $r->senderEmail = $reply->useremail;
                $r->senderName = $reply->username;
                $r->timestamp = $reply->tick;

                $r->location = $reply->location;
                $r->locname = $reply->locname;
                $r->lat = $reply->lat;
                $r->lng = $reply->lng;
                $r->provider = $reply->provider;
                $r->accuracy = $reply->accuracy;

                return new ReplyImpl(reply);
            } elseif ($textObj->type === 'primary' && $textObj->latestAction === 'reply') {
                $reply = $textObj->details->detitem->resps[0];
                $action = $textObj->actions->actitem[0];
                $fieldKeys = $this->extractKeys($action);
                $fieldValues = $reply->values;

                $r = new \stdClass();
                $r->formid = $textObj->formid;
                $r->sender = $reply->user->userid;
                $r->senderEmail = $reply->user->useremail;
                $r->senderName = $reply->user->username;
                $r->timestamp = $reply->timestamp;
                $r->actionreplace = $textObj->actionreplace;
                for ($i = 0; $i < count($fieldKeys); ++$i) {
                    $r->{$fieldKeys[$i]} = $fieldValues[$i];
                }

                return new Messages\Reply($r);
            }
        } catch (Exception $ex) {
        }
    }

    public function currentRoom()
    {
        return new Room($this->activity->roomId);
    }

    public function currentRoomAlias()
    {
        return;
    }

    public function currentSender()
    {
        return $this->activity->sender;
    }

    public function currentSenderId()
    {
        return $this->activity->sender->id;
    }

    public function p2p($email)
    {
        return new Room($email);
    }

    public function raw($message)
    {
        return new IncomingChatlet($message);
    }

    public function recursiveIterate($relation)
    {
        return;
    }

    public function relatedRooms($relation)
    {
        return;
    }
}
