<?php

namespace TeamChat\SDK\Data;

use PDO;

/**
 * Description of PDOPersistenceLayer.
 *
 * @author Team Chat <support@teamchat.com>
 */
class PDOPersistenceLayer implements PersistenceLayerInterface
{
    /**
     * @var PDO
     */
    public $pdo;

    public $botemail;

    const TABLE_NAME = 'botdata';

    public function __construct(PDO $pdo, $botemail)
    {
        $this->pdo = $pdo;
        $this->botemail = $botemail;
        $this->ensureBot();
    }

    public function config()
    {
        $stmt = $this->pdo->prepare('
            SELECT config FROM '.self::TABLE_NAME.'
            WHERE email = ?
        ');
        $stmt->execute(array($this->botemail));
        $row = $stmt->fetch(PDO::FETCH_ASSOC);

        return json_decode($row['config']);
    }

    public function data()
    {
        $stmt = $this->pdo->prepare('
            SELECT data FROM '.self::TABLE_NAME.'
            WHERE email = ?
        ');
        $stmt->execute(array($this->botemail));
        $row = $stmt->fetch(PDO::FETCH_ASSOC);

        return json_decode($row['data']);
    }

    public function updateConfig($config)
    {
        $stmt = $this->pdo->prepare('
            UPDATE '.self::TABLE_NAME.'
            SET config = ?
            WHERE email = ?
        ');

        return $stmt->execute(array(json_encode($config), $this->botemail));
    }

    public function updateData($data)
    {
        $stmt = $this->pdo->prepare('
            UPDATE '.self::TABLE_NAME.'
            SET data = ?
            WHERE email = ?
        ');

        return $stmt->execute(array(json_encode($data), $this->botemail));
    }

    public function ensureBot()
    {
        $this->ensureTable();
        //TODO - This is sqlite specific entry. Change it to be more generic.
        $stmt = $this->pdo->prepare('
            INSERT OR IGNORE INTO '.self::TABLE_NAME.'
            (email, config, data) VALUES (?,?,?)
        ');
        $stmt->execute(array($this->botemail, '{}', '{}'));
    }

    public function ensureTable()
    {
        $this->pdo->exec('
            CREATE TABLE IF NOT EXISTS
            '.self::TABLE_NAME.'
            (
                email VARCHAR(255) NOT NULL,
                config MEDIUMTEXT,
                data MEDIUMTEXT,
                PRIMARY KEY (`email`)
            )
        ');
    }
}
