<?php

namespace TeamChat\SDK\Data;

/**
 * Description of PDOUserData.
 *
 * @author Team Chat <support@teamchat.com>
 */
class Record implements UserDataInterface, BotDataInterface
{
    const KEY_EMAIL = 'email';
    const KEY_PASSWORD = 'password';
    const KEY_TOKEN = 'token';
    const KEY_LAST_MSG_TIMESTAMP = 'lastMsgTimestamp';
    const KEY_USERID = 'userId';
    const KEY_LAST_FORM_ID = 'lastformid';
    const KEY_LAST_MSG_ID = 'lastmsgid';
    const KEY_ALIASES = 'aliases';
    const KEY_FLOWLETS = 'flowlets';
    const KEY_FLOWLET_FORM_MAP = 'flowletFormMap';
    const KEY_FLOWLET_ROOM_MAP = 'flowletRoomMap';

    /**
     * @var PersistenceLayerInterface
     */
    public $layer;

    public function __construct(PersistenceLayerInterface $layer)
    {
        $this->layer = $layer;
    }

    private function getConfigParam($key)
    {
        $config = $this->layer->config();
        if (property_exists($config, $key)) {
            return $config->{$key};
        }

        return;
    }

    public function formidsByAlias($alias)
    {
        $formids = array();
        $aliases = $this->getConfigParam(self::KEY_ALIASES);
        if (is_null($aliases)) {
            return $formids;
        }
        foreach ($aliases as $formid => $as) {
            if (in_array($alias, $as)) {
                $formids[] = $formid;
            }
        }

        return $formids;
    }

    public function getAlias($formid)
    {
        $aliases = $this->getConfigParam(self::KEY_ALIASES);
        if (is_null($aliases)) {
            return array();
        }
        if (property_exists($aliases, $formid)) {
            return $aliases->{$formid};
        }

        return array();
    }

    public function getEmail()
    {
        return $this->getConfigParam(self::KEY_EMAIL);
    }

    public function getFlowletData($flowlet)
    {
        $flowlets = $this->getConfigParam(self::KEY_FLOWLETS);
        if (is_null($flowlets)) {
            return;
        }
        if (property_exists($flowlets, $flowlet)) {
            return $flowlets->{$flowlet};
        }

        return;
    }

    public function getFlowletForFormid($formid)
    {
        $map = $this->getConfigParam(self::KEY_FLOWLET_FORM_MAP);
        if (is_null($map)) {
            return;
        }
        if (property_exists($map, $formid)) {
            return $map->{$formid};
        }

        return;
    }

    public function getFlowletForRoom($roomid)
    {
        $map = $this->getConfigParam(self::KEY_FLOWLET_ROOM_MAP);
        if (is_null($map)) {
            return;
        }
        if (property_exists($map, $roomid)) {
            return $map->{$roomid};
        }

        return;
    }

    public function getLastMsgId()
    {
        return $this->getConfigParam(self::KEY_LAST_MSG_ID);
    }

    public function getLastMsgTimestamp()
    {
        return $this->getConfigParam(self::KEY_LAST_MSG_TIMESTAMP);
    }

    public function getPassword()
    {
        return $this->getConfigParam(self::KEY_PASSWORD);
    }

    public function getToken()
    {
        return $this->getConfigParam(self::KEY_TOKEN);
    }

    public function getUserId()
    {
        return $this->getConfigParam(self::KEY_USERID);
    }

    public function updateAlias($formid, $alias)
    {
        $config = $this->layer->config();
        if (!property_exists($config, self::KEY_ALIASES)) {
            $config->{self::KEY_ALIASES} = new \stdClass();
        }
        if (!property_exists($config->{self::KEY_ALIASES}, $formid)) {
            $config->{self::KEY_ALIASES}->{$formid} = array();
        }
        if (!in_array($alias, $config->{self::KEY_ALIASES}->{$formid})) {
            $config->{self::KEY_ALIASES}->{$formid}[] = $alias;
        }
        $this->layer->updateConfig($config);
    }

    public function updateEmail($email)
    {
        $config = $this->layer->config();
        $config->{self::KEY_EMAIL} = $email;
        $this->layer->updateConfig($config);
    }

    public function updateFlowletData($id, $data)
    {
        $config = $this->layer->config();
        if (!property_exists($config, self::KEY_FLOWLETS)) {
            $config->{self::KEY_FLOWLETS} = new \stdClass();
        }
        $config->{self::KEY_FLOWLETS}->{$id} = $data;
        $this->layer->updateConfig($config);
    }

    public function updateFlowletFormidMapping($formid, $flowletid)
    {
        $config = $this->layer->config();
        if (!property_exists($config, self::KEY_FLOWLET_FORM_MAP)) {
            $config->{self::KEY_FLOWLET_FORM_MAP} = new \stdClass();
        }
        $config->{self::KEY_FLOWLET_FORM_MAP}->{$formid} = $flowletid;
        $this->layer->updateConfig($config);
    }

    public function updateFlowletRoomidMapping($roomid, $flowletid)
    {
        $config = $this->layer->config();
        if (!property_exists($config, self::KEY_FLOWLET_ROOM_MAP)) {
            $config->{self::KEY_FLOWLET_ROOM_MAP} = new \stdClass();
        }
        $config->{self::KEY_FLOWLET_ROOM_MAP}->{$roomid} = $flowletid;
        $this->layer->updateConfig($config);
    }

    public function updateLastMsgId($msgid)
    {
        $config = $this->layer->config();
        $config->{self::KEY_LAST_MSG_ID} = $msgid;
        $this->layer->updateConfig($config);
    }

    public function updateLastMsgTimestamp($lastMsgTimestamp)
    {
        $config = $this->layer->config();
        if (property_exists($config, self::KEY_LAST_MSG_TIMESTAMP) && $config->{self::KEY_LAST_MSG_TIMESTAMP} >= $lastMsgTimestamp) {
            return;
        }
        $config->{self::KEY_LAST_MSG_TIMESTAMP} = $lastMsgTimestamp;
        $this->layer->updateConfig($config);
    }

    public function updatePassword($password)
    {
        $config = $this->layer->config();
        $config->{self::KEY_PASSWORD} = $password;
        $this->layer->updateConfig($config);
    }

    public function updateToken($token)
    {
        $config = $this->layer->config();
        $config->{self::KEY_TOKEN} = $token;
        $this->layer->updateConfig($config);
    }

    public function updateUserId($userId)
    {
        $config = $this->layer->config();
        $config->{self::KEY_USERID} = $userId;
        $this->layer->updateConfig($config);
    }

    public function addField($section, $field, $val)
    {
        if (is_null($field) || is_null($section)) {
            return;
        }
        $data = $this->data();
        if (!property_exists($data, $section)) {
            $data->{$section} = new \stdClass();
        }
        $data->{$section}->{$field} = $val;
        $this->layer->updateData($data);
    }

    public function addRow($section, $fields)
    {
        $data = $this->data();
        if (!property_exists($data, $section)) {
            $data->{$section} = new \stdClass();
        }
        if (!property_exists($data->{section}, 'table')) {
            $data->{$section}->table = array();
        }
        $data->{$section}->table[] = $fields;
        $this->layer->updateData($data);
    }

    public function addToField($section, $field, $val)
    {
        $data = $this->data();
        if (!property_exists($data, $section)) {
            $data->{$section} = new \stdClass();
        }
        if (!property_exists($data->{$section}, $field) || !is_int($data->{$section}->{$field})) {
            $data->{$section}->{$field} = 0;
        }
        $data->{$section}->{$field} += $val;
        $this->layer->updateData($data);
    }

    public function data()
    {
        return $this->layer->data();
    }

    public function getField($section, $field)
    {
        if (is_null($field) || is_null($section)) {
            return;
        }
        $data = $this->data();
        if (!property_exists($data, $section) || !property_exists($data->{$section}, $field)) {
            return;
        }

        return $data->{$section}->{$field};
    }

    public function getRows($section)
    {
        $rows = $this->getField($section, 'table');
        if (empty($section) || empty($rows)) {
            return array();
        }

        return $rows;
    }

    public function remove($section)
    {
        $data = $this->data();
        if (property_exists($data, $section)) {
            unset($data->{$section});
            $this->layer->updateData($data);
        }
    }

    public function removeField($section, $fieldName)
    {
        if (empty($section) || empty($fieldName)) {
            return;
        }
        $data = $this->data();
        if (property_exists($data, $section) && property_exists($data->{$section}, $fieldName)) {
            unset($data->{$section}->{$fieldName});
            $count = count(get_object_vars($data->{$section}));
            if ($count === 0) {
                unset($data->{$section});
            }
            $this->layer->updateData($data);
        }
    }
}
