<?php

namespace TeamChat\SDK\Messages\Chatlets;

use stdClass;
use TeamChat\SDK\Form;

/**
 * Description of PrimaryChatlet.
 *
 * @author Team Chat <support@teamchat.com>
 */
class PrimaryChatlet extends Chatlet
{
    const OPERATION_SUM = 'sum';
    const OPERATION_AVERAGE = 'average';

    public $content;
    public $summary;
    public $details;
    public $comments;
    public $actions;
    public $actitem;
    public $form;
    public $resp;

    public function __construct()
    {
        $this->chatlet = json_decode($this->getBasicTemplate());
        $this->loadSections();
    }

    public function loadSections()
    {
        $this->content = $this->chatlet->content;
        $this->summary = $this->chatlet->summdesc;
        $this->details = $this->chatlet->detdesc;
        $this->comments = $this->chatlet->comments;
        $this->actions = $this->chatlet->actions;
        $this->actitem = array();
        $this->form = new stdClass();
    }

    public function allowComments($flag)
    {
        if ($flag) {
            $this->comments->addcomment = 'true';
        } else {
            $this->comments->addcomment = 'false';
        }

        return $this;
    }

    public function setQuestion($question)
    {
        $this->content->text = $question;

        return $this;
    }

    public function setQuestionHtml($question)
    {
        $this->content->html = $question;

        return $this;
    }

    public function setReplyScreen(Form $replyScreen)
    {
        $this->resp = $replyScreen->getRespDefn();
        $this->details->fields = $replyScreen->getDetailDefn()->fields;
        $fields = $replyScreen->getFormDefn()->fields;
        $this->checkAndAddButtonAction();
        $this->form->imgUrl = '';
        $this->form->type = 'meta';
        $this->form->fields = $fields;

        return $this;
    }
    public function setReplyLabel($replyLabel)
    {
        $this->checkAndAddButtonAction();
        if (!is_null($this->actitem)) {
            $this->actitem[0]->button->label = $replyLabel;
        }
        $this->form->imgUrl = '';
        $this->form->type = 'meta';
        $this->form->displayname = $replyLabel;

        return $this;
    }

    public function setDetailsLabel($detailsLabel)
    {
        $this->details->label = $detailsLabel;

        return $this;
    }

    public function showDetails($show)
    {
        if ($show) {
            $this->chatlet->detailsLinkVisibility = 'all';
        } else {
            $this->chatlet->detailsLinkVisibility = 'hidden';
        }

        return $this;
    }

    public function addSummary($field, $operation)
    {
        if ($operation == self::OPERATION_SUM) {
            $op = 'Sum';
            $display = 'Total';
        } elseif ($operation == self::OPERATION_AVERAGE) {
            $op = 'Avg';
            $display = 'Average';
        } else {
            return $this;
        }

        $fields = array();
        if (property_exists($this->summary, 'fields')) {
            $fields = $this->summary->fields;
        }

        $fld = new stdClass();
        $fld->operation = op;
        $fld->drilldown = '';
        $fld->name = $field + $op;
        $fld->display = $display + ' (@{' + $field + '})';
        $fld->fieldname = $field;
        $fields[] = $fld;

        return $this;
    }
    public function setReplyAction($label, $key, $val)
    {
        $respjson = new stdClass();
        $respjson->$key = $val;
        if (!is_null($this->actitem) && count($this->actitem) > 0) {
            return $this;
        }

        $actionitem = json_decode('{"button": {"resptype": "sendmsg","label": "'.$label.'"}}');
        $action = $actionitem->button;
        $action->deffields = $respjson;
        if (is_null($this->actitem)) {
            $this->actitem = array();
        }
        $this->actitem[] = $actionitem;
        $this->actions->actitem = $actitem;

        $field = new stdClass();
        $field->display = '';
        $field->key = $key;
        $this->details->fields[] = $field;

        return $this;
    }

    public function addReplyAction($label, $key, $val)
    {
        $respjson = new stdClass();
        $respjson->$key = $val;

        $actionitem = json_decode('{"button": {"resptype": "sendmsg","label": "'.$label.'"}}');
        $action = $actionitem->button;
        $action->deffields = $respjson;
        if (is_null($this->actitem)) {
            $this->actitem = array();
        }
        $this->actitem[] = $actionitem;
        $this->actions->actitem = $actitem;

        $field = new stdClass();
        $field->display = '';
        $field->key = $key;
        $this->details->fields[] = $field;

        return $this;
    }

    public function raw()
    {
        if (!is_null($this->actitem) && count($this->actitem) > 0 && property_exists($this->actitem[0], 'button') && property_exists($this->form, 'fields')) {
            $this->actitem[0]->button->formjson = json_encode($this->form);
            $this->actitem[0]->button->respjson = json_encode($this->resp);
        }

        $raw = json_encode($this->chatlet);

        return $raw;
    }

    private function checkAndAddButtonAction()
    {
        if (!is_null($this->actitem) && count($this->actitem) == 0) {
            $button = json_decode('{"button": {"resptype": "compose","label": "Reply", "formjson":"","respjson":""}}');
            $this->actitem[] = $button;
            $this->actions->actitem = $this->actitem;
        }
    }

    public function getBasicTemplate()
    {
        return '
            {
                "imgUrl": "",
                "type": "primary",
                "displayname": "Primary Chatlet",
                "content": {
                    "text": "",
                    "html": ""
                },
                "xslt": "'.self::XSLT_VERSION.'",
                "summdesc": {
                    "fields": []
                },
                "detailsLinkVisibility": "hidden",
                "detdesc": {
                    "label": "Replies",
                    "showcount": "true",
                    "fields": []
                },
                "actionreplace": "replace",
                "actions": {
                    "actitem": [
                    ]
                },
                "comments": {
                    "addcomment": "false"
                }
            }
        ';
    }
}
